import { world, system, EquipmentSlot, EntityEquippableComponent, PlayerJoinAfterEvent } from "@minecraft/server";
import Vanilla from "./data/vanilla.js";
import LightHandler from "./lib/lightHandler.js";

const handlers = new Map();

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    const equippableComponent = player.getComponent(EntityEquippableComponent.componentId);
    const mainHand = equippableComponent.getEquipment(EquipmentSlot.Mainhand);

    if (!handlers.has(player.id)) handlers.set(player.id, new LightHandler(player));
    if (handlers.has(player.id)) {
      const handler = handlers.get(player.id);
      handler.removeUnused();
      handler.removeUnused(JSON.parse(handler.getLastCoordinates()).center);
    }
    if (!mainHand) continue;
    let itemLight = Vanilla[mainHand.typeId ?? ""];

    mainHand.typeId.includes("candle") ? itemLight = 8 : false;
    mainHand.getTags()?.includes("dynamic_light") ? itemLight = 12 : false;
    
    if (!itemLight) continue;
    const handler = handlers.get(player.id);

    handler.removeUnused();
    handler.deleteLastCoordinates();
    handler.iluminate(itemLight);
    handler.saveLastCoordinates();
  }
}, 0);