import { BlockPermutation } from "@minecraft/server";

export default class LightHandler {
    constructor(player) {
        this.lightBlocks = [];
        this.player = player;
    }

    iluminate(level) {
        for (let y = 0; y < 3; y++) {
            const lightBlockPosition = this._offset(this.player.location, { x: 0, y, z: 0 });
            const block = this.player.dimension.getBlock(lightBlockPosition);

            if (!block || !block.isAir) continue;

            block.setPermutation(BlockPermutation.resolve("minecraft:light_block", {
                "block_light_level": level
            }));
            this.lightBlocks.push(block);
        }
    }

    removeUnused(center = null) {
        if (this.lightBlocks.length < 1 && !center) return;
        for (let y = 0; y < 3; y++) {
            try {
                if (center) {
                    const lightBlockPosition = this._offset(center, { x: 0, y, z: 0 });
                    const block = this.player
                        .dimension
                        .getBlock(lightBlockPosition);
                        
                    if (!block.permutation.matches("minecraft:light_block")) continue;
                    block.setPermutation(BlockPermutation.resolve("minecraft:air"));
                    continue;
                }
                const lightBlockPosition = this._offset(this.player.location, { x: 0, y, z: 0 });

                if (!this.lightBlocks[y]) continue
                if (this._equalVector(this.lightBlocks[y]?.location, lightBlockPosition)) continue;
                this.lightBlocks[y].setPermutation(BlockPermutation.resolve("minecraft:air"));
                this.lightBlocks.splice(y, 1);
            } catch (e) {
                this.lightBlocks = [];
            }
        }
    }

    reloadLights(lightLevel) {
        this.removeUnused();
        this.iluminate(lightLevel);
    }

    _equalVector(vec1, vec2) {
        function _reduceVector(vector) {
            return Object.values(vector).reduce((backValue, value) => { return backValue + value }, 0);
        }
        return (_reduceVector(vec1) - _reduceVector(vec2)) == 0;
    }

    _offset(vec1, vec2) {
        return {
            x: (vec1.x < 0 && vec2.x < 0) ? vec1.x - vec2.x : vec1.x + vec2.x,
            y: (vec1.y < 0 && vec2.y < 0) ? vec1.y - vec2.y : vec1.y + vec2.y,
            z: (vec1.z < 0 && vec2.z < 0) ? vec1.z - vec2.z : vec1.z + vec2.z
        };
    }

    saveLastCoordinates() {
        this.deleteLastCoordinates();
        this.player.addTag(`{ "center": ${JSON.stringify(this.player.location)} }`);
    }

    getLastCoordinates() {
        return this.player
            .getTags()
            .find(tag => {
                try {
                    JSON.parse(tag)?.center;
                    return true
                } catch (e) {
                    return false;
                }
            }) ?? `{ "center": ${JSON.stringify(this.player.location)} }`;
    }

    deleteLastCoordinates() {
        if (!this.getLastCoordinates()) return;
        this.player.removeTag(this.getLastCoordinates())
    }
}